#region Copyright information
// 
// Copyright  2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: IClusterer.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-12-08 8:26 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using BOL.Maths.Distributions;

namespace BOL.Algorithms.Clustering
{
    public interface IClusterer<in TPoint>
        where TPoint: struct
    {
        Func<IEnumerable<TPoint>, IEnumerable<TPoint>, double> DistanceFunction { get; }
        IEnumerable<IGrouping<int, TSource>> Cluster<TSource>(IEnumerable<TSource> sources, Func<TSource, TPoint[]> pointSelector);
    }

    public interface IConnectivityClusterer<in TPoint> : IClusterer<TPoint>
        where TPoint : struct
    {
        new DendrogramNode<TSource> Cluster<TSource>(IEnumerable<TSource> sources, Func<TSource, TPoint[]> pointSelector);
    }

    public interface ICentroidClusterer : IClusterer<double>
    {
        IList<double[]> Centroids { get; }
        int K { get; }
    }

    public interface IDistributionClusterer : IClusterer<double>
    {
        IMultivariateDistribution<double> Distribution { get; }
    }

    public interface IDensityClusterer : IClusterer<double>
    {
        IList<Density> Centroids { get; }
    }

    public interface IMapClusterer : IClusterer<double>
    {
        double[][] Map { get; }
        int GridSize { get; }
    }
}
