﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: Secant.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2013-01-24 4:34 PM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;

namespace BOL.Algorithms.Optimization
{
    public sealed class Secant : RootFinderByValueBase
    {
        #region Constructor

        public Secant(Func<double, double> function, int numberOfIterations = 100, double tolerance = 1e-10, bool isVerbose = true)
            : base(function, numberOfIterations, tolerance, isVerbose) { }

        #endregion

        #region RootFinderByValueBase overriden

        protected override void Expect(double min, double max)
        {
            var fMax = Function(max);

            Current = max - fMax * (min - max) / (Function(min) - fMax);
            Value = Function(Current.Value);
        }

        protected override void Maximize(ref double min, ref double max)
        {
            min = max;
            if (Current.HasValue)
                max = Current.Value;
        }

        #endregion
    }
}
