#region Copyright information
// 
// Copyright  2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: MarkovChain.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2013-01-24 4:34 PM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using BOL.Maths.Distributions;

namespace BOL.Algorithms.StateMachines
{
    public class MarkovChain<TState> : MarkovModelBase<TState>, ICloneable
    {
        #region Constructors

        public MarkovChain(IDictionary<TState, double> initialDistribution, IDictionary<TState, IDistribution<TState>> transitionDistribution)
            : base(initialDistribution, transitionDistribution) { }

        public MarkovChain(ICollection<TState> states, IDictionary<TState, IDistribution<TState>> transitionDistribution)
            : this(states.ToDictionary(s => s, s => 1.0 / states.Count()), transitionDistribution) { }

        #endregion

        #region ICloneable implementation

        public MarkovChain<TState> Clone()
        {
            return new MarkovChain<TState>(
                InitialDistribution.ToDictionary(x => x.Key, x => x.Value),
                TransitionDistribution.ToDictionary(x => x.Key, x => x.Value.Clone() as IDistribution<TState>)
                );
        }

        object ICloneable.Clone()
        {
            return Clone();
        }

        #endregion
    }
}
