#region Copyright information
// 
// Copyright  2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: DelimitedTextWriter.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-20 9:56 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.IO;

namespace BOL.IO
{
    public class DelimitedTextWriter<T> : WriterBase
    {
        #region Private variables

        private readonly string _delimiter;
        private readonly IEnumerable<IEnumerable<T>> _source;

        #endregion

        #region Constructor

        public DelimitedTextWriter(IEnumerable<IEnumerable<T>> source, Stream stream, string delimiter = "\t")
            :base(stream)
        {
            _source = source;
            _delimiter = delimiter;
        }

        public DelimitedTextWriter(IEnumerable<IEnumerable<T>> source, string path, string delimiter = "\t")
            : this(source, new FileStream(path, FileMode.Create), delimiter) { }
        
        #endregion

        #region IWriter implementation

        public override bool Write()
        {
            try
            {
                foreach (var row in _source)
                    StreamWriter.WriteLine(String.Join(_delimiter, row));
            }
            catch (Exception ex)
            {
                Console.WriteLine(ex.Message);
                return false;
            }

            return true;
        }

        #endregion
    }
}
