﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: AbsoluteNorm.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Descriptive
{
    public static partial class Descriptive
    {
        public static decimal AbsoluteNorm(this IEnumerable<decimal> source)
        {
            return source.Aggregate(Decimal.Zero, (norm, next) => norm + Math.Abs(next));
        }

        public static double AbsoluteNorm(this IEnumerable<double> source)
        {
            return source.Aggregate(0.0, (norm, next) => norm + Math.Abs(next));
        }
        
        public static float AbsoluteNorm(this IEnumerable<float> source)
        {
            return source.Aggregate(0.0f, (norm, next) => norm + Math.Abs(next));
        }

        public static double AbsoluteNorm(this IEnumerable<int> source)
        {
            return source.Aggregate(0, (norm, next) => norm + Math.Abs(next));
        }

        public static double AbsoluteNorm(this IEnumerable<long> source)
        {
            return source.Aggregate(0L, (norm, next) => norm + Math.Abs(next));
        }

        public static decimal AbsoluteNorm<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).AbsoluteNorm();
        }

        public static double AbsoluteNorm<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).AbsoluteNorm();
        }

        public static float AbsoluteNorm<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).AbsoluteNorm();
        }

        public static double AbsoluteNorm<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).AbsoluteNorm();
        }

        public static double AbsoluteNorm<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).AbsoluteNorm();
        }
    }
}
