﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: CoefficientOfVariation.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Descriptive
{
    public static partial class Descriptive
    {
        public static decimal CoefficientOfVariation(this IEnumerable<decimal> source)
        {
            var list = source.ToList();

            return list.StandardDeviation() / list.Average();
        }

        public static double CoefficientOfVariation(this IEnumerable<double> source)
        {
            var list = source.ToList();

            return list.StandardDeviation() / list.Average();
        }

        public static float CoefficientOfVariation(this IEnumerable<float> source)
        {
            var list = source.ToList();

            return list.StandardDeviation() / list.Average();
        }

        public static double CoefficientOfVariation(this IEnumerable<int> source)
        {
            var list = source.ToList();

            return list.StandardDeviation() / list.Average();
        }

        public static double CoefficientOfVariation(this IEnumerable<long> source)
        {
            var list = source.ToList();

            return list.StandardDeviation() / list.Average();
        }

        public static decimal CoefficientOfVariation<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).CoefficientOfVariation();
        }

        public static double CoefficientOfVariation<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).CoefficientOfVariation();
        }

        public static float CoefficientOfVariation<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).CoefficientOfVariation();
        }
        
        public static double CoefficientOfVariation<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).CoefficientOfVariation();
        }
        
        public static double CoefficientOfVariation<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).CoefficientOfVariation();
        }
    }
}
