﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: Median.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Descriptive
{
    public static partial class Descriptive
    {
        public static decimal Median(this IEnumerable<decimal> source)
        {
            return (decimal)source.Select(x => (double) x).Median();
        }

        public static double Median(this IEnumerable<double> source)
        {
            var sortedList = source.OrderBy(x => x).ToList();

            var count = sortedList.Count();
            var index = count / 2;

            return count % 2 == 0 ? (sortedList.ElementAt(index) + sortedList.ElementAt(index - 1)) / 2 : sortedList.ElementAt(index);
        }

        public static float Median(this IEnumerable<float> source)
        {
            return (float)source.Select(x => (double)x).Median();
        }

        public static int Median(this IEnumerable<int> source)
        {
            return (int)source.Select(x => (double)x).Median();
        }

        public static long Median(this IEnumerable<long> source)
        {
            return (long)source.Select(x => (double)x).Median();
        }

        public static decimal Median<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).Median();
        }

        public static double Median<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).Median();
        }

        public static float Median<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).Median();
        }

        public static int Median<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).Median();
        }

        public static long Median<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).Median();
        }
    }
}
