﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: StandardDeviationP.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Descriptive
{
    public static partial class Descriptive
    {
        public static decimal StandardDeviationP(this IEnumerable<decimal> source)
        {
            return (decimal)source.Select(x => (double)x).StandardDeviationP();
        }

        public static double StandardDeviationP(this IEnumerable<double> source)
        {
            var list = source.ToList();
            var mean = list.Average();

            return Math.Sqrt(list.Aggregate(0.0, (sum, x) => sum + (x - mean) * (x - mean)) / list.Count());
        }

        public static float StandardDeviationP(this IEnumerable<float> source)
        {
            return (float)source.Select(x => (double)x).StandardDeviationP();
        }

        public static double StandardDeviationP(this IEnumerable<int> source)
        {
            return source.Select(x => (double)x).StandardDeviationP();
        }

        public static double StandardDeviationP(this IEnumerable<long> source)
        {
            return source.Select(x => (double)x).StandardDeviationP();
        }
 
        public static decimal StandardDeviationP<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).StandardDeviationP();
        }

        public static double StandardDeviationP<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).StandardDeviationP();
        }

        public static float StandardDeviationP<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).StandardDeviationP();
        }

        public static double StandardDeviationP<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).StandardDeviationP();
        }

        public static double StandardDeviationP<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).StandardDeviationP();
        }
    }
}
