﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: StandardError.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Descriptive
{
    public static partial class Descriptive
    {
        public static decimal StandardError(this IEnumerable<decimal> source)
        {
            return (decimal)source.Select(x => (double)x).StandardError();
        }

        public static double StandardError(this IEnumerable<double> source)
        {
            var list = source.ToList();
            var avg = list.Average();
            var d = list.Aggregate(0.0, (total, next) => Math.Pow(next - avg, 2));

            return Math.Sqrt(d / (list.Count() - 1) / list.Count());
        }

        public static float StandardError(this IEnumerable<float> source)
        {
            return (float)source.Select(x => (double)x).StandardError();
        }

        public static double StandardError(this IEnumerable<int> source)
        {
            return source.Select(x => (double)x).StandardError();
        }

        public static double StandardError(this IEnumerable<long> source)
        {
            return source.Select(x => (double)x).StandardError();
        }

        public static decimal StandardError<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).StandardError();
        }

        public static double StandardError<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).StandardError();
        }

        public static float StandardError<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).StandardError();
        }

        public static double StandardError<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).StandardError();
        }

        public static double StandardError<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).StandardError();
        }
    }
}
