﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: BrayCurtis.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Distances
{
    public static partial class Disimilarity
    {
        public static double BrayCurtis(this IEnumerable<bool> first, IEnumerable<bool> second)
        {
            var bc = first.BinaryCardinality(second);
            var bc2 = bc.Item2 + bc.Item3;

            return bc2 / (double)(2 * bc.Item1 + bc2);
        }

        public static double BrayCurtis(this IEnumerable<double> first, IEnumerable<double> second)
        {
            var x = first.ToList();
            var y = second.ToList();

            var numerator = x.Zip(y, (xc, yc) => Math.Abs(xc - yc)).Sum();
            var denominator = x.Zip(y, (xc, yc) => xc + yc).Sum();

            return numerator / denominator;
        }
    }

    public static partial class Similarity
    {
        public static double BrayCurtis(this IEnumerable<double> first, IEnumerable<double> second)
        {
            var x = first.ToList();
            var y = second.ToList();

            var length = Math.Min(x.Count, y.Count);
            var xAvg = x.Average();
            var yAvg = y.Average();
            var minSum = x.Zip(y, Math.Min).Sum();

            return 2 * minSum / length / (xAvg + yAvg);
        }
    }
}
