﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: Kulczynski.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Distances
{
    public static partial class Similarity
    {
        /// <summary>
        ///                 a
        /// Kulczynski1 = -----
        ///               b + c
        /// </summary>
        public static double Kulczynski1(this IEnumerable<bool> first, IEnumerable<bool> second)
        {
            var bc = first.BinaryCardinality(second);

            return bc.Item1 / (double)(bc.Item2 + bc.Item3);
        }

        /// <summary>
        ///                  a * (2a + b + c)
        /// Kulczynski2 = ---------------------
        ///               2 * (a + b) * (a + c)
        /// </summary>
        public static double Kulczynski2(this IEnumerable<bool> first, IEnumerable<bool> second)
        {
            var bc = first.BinaryCardinality(second);

            return 0.5 * bc.Item1 * (2 * bc.Item1 + bc.Item2 + bc.Item3) / ((bc.Item1 + bc.Item2) * (bc.Item1 + bc.Item3));
        }

        public static double Kulczynski(this IEnumerable<double> first, IEnumerable<double> second)
        {
            var x = first.ToList();
            var y = second.ToList();

            return x.Zip(y, Math.Min).Sum() / x.Zip(y, (xc, yc) => Math.Abs(xc - yc)).Sum();
        }
    }
}
