﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: Pearson.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using BOL.Linq.Descriptive;

namespace BOL.Linq.Distances
{
    public static partial class Similarity
    {
        /// <summary>
        ///                        ad - bc
        /// Pearson = ----------------------------------
        ///           sqrt((a + b)(c + d)(a + c)(b + d))
        /// </summary>
        public static double Pearson(this IEnumerable<bool> first, IEnumerable<bool> second)
        {
            var bc = first.BinaryCardinality(second);

            return (bc.Item1 * bc.Item4 - bc.Item2 * bc.Item3) / Math.Sqrt((bc.Item1 + bc.Item2) * (bc.Item3 + bc.Item4) * (bc.Item1 + bc.Item3) * (bc.Item2 + bc.Item4));
        }

        public static double Pearson(this IEnumerable<double> first, IEnumerable<double> second)
        {
            var x = first.ToList();
            var y = second.ToList();
            var length = x.Count;

            var xMean = x.Average();
            var yMean = y.Average();
            var xStdDev = x.StandardDeviation();
            var yStdDev = y.StandardDeviation();

            return x.Zip(y, (xc, yc) => (xc - xMean) * (yc - yMean)).Sum() / (length * xStdDev * yStdDev);
        }
    }
}
