﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: Product.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-10-16 8:53 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq
{
    public static partial class Linq
    {
        public static decimal Product(this IEnumerable<decimal> source)
        {
            return source.Aggregate(1M, (product, next) => product * next);
        }

        public static double Product(this IEnumerable<double> source)
        {
            return source.Aggregate(1.0, (product, next) => product * next);
        }
        
        public static float Product(this IEnumerable<float> source)
        {
            return source.Aggregate(1.0f, (product, next) => product * next);
        }

        public static int Product(this IEnumerable<int> source)
        {
            return source.Aggregate(1, (product, next) => product * next);
        }

        public static long Product(this IEnumerable<long> source)
        {
            return source.Aggregate(1L, (product, next) => product * next);
        }

        public static decimal Product<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).Product();
        }

        public static double Product<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).Product();
        }

        public static float Product<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).Product();
        }

        public static int Product<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).Product();
        }

        public static long Product<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).Product();
        }
    }
}
