﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: DivideByMax.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Transformation
{
    public static partial class Scaling
    {
        public static IEnumerable<decimal> DivideByMax(this IEnumerable<decimal> source)
        {
            var list = source.ToList();
            var max = Math.Abs(list.Max());
            if (max == 0M)
                throw new DivideByZeroException();

            return list.Scale(1 / max);
        }

        public static IEnumerable<double> DivideByMax(this IEnumerable<double> source)
        {
            var list = source.ToList();
            var max = Math.Abs(list.Max());
            if (max.Equals(0.0))
                throw new DivideByZeroException();

            return list.Scale(1 / max);
        }

        public static IEnumerable<float> DivideByMax(this IEnumerable<float> source)
        {
            var list = source.ToList();
            var max = Math.Abs(list.Max());
            if (max.Equals(0.0F))
                throw new DivideByZeroException();

            return list.Scale(1 / max);
        }

        public static IEnumerable<int> DivideByMax(this IEnumerable<int> source)
        {
            var list = source.ToList();
            var max = Math.Abs(list.Max());
            if (max == 0)
                throw new DivideByZeroException();

            return list.Scale(1 / max);
        }

        public static IEnumerable<long> DivideByMax(this IEnumerable<long> source)
        {
            var list = source.ToList();
            var max = Math.Abs(list.Max());
            if (max == 0L)
                throw new DivideByZeroException();

            return list.Scale(1 / max);
        }

        public static IEnumerable<decimal> DivideByMax<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).DivideByMax();
        }

        public static IEnumerable<double> DivideByMax<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).DivideByMax();
        }

        public static IEnumerable<float> DivideByMax<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).DivideByMax();
        }

        public static IEnumerable<int> DivideByMax<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).DivideByMax();
        }

        public static IEnumerable<long> DivideByMax<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).DivideByMax();
        }
    }
}
