﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: ImputeWithMean.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Transformation
{
    public static partial class Imputation
    {
        public static IEnumerable<decimal> ImputeWithMean(this IEnumerable<decimal?> source)
        {
            var list = source.ToList();
            var mean = list.Coalesce().Average();

            return list.ImputeWithConstant(mean);
        }

        public static IEnumerable<double> ImputeWithMean(this IEnumerable<double?> source)
        {
            var list = source.ToList();
            var mean = list.Coalesce().Average();

            return list.ImputeWithConstant(mean);
        }

        public static IEnumerable<float> ImputeWithMean(this IEnumerable<float?> source)
        {
            var list = source.ToList();
            var mean = list.Coalesce().Average();

            return list.ImputeWithConstant(mean);
        }

        public static IEnumerable<int> ImputeWithMean(this IEnumerable<int?> source)
        {
            var list = source.ToList();
            var mean = list.Coalesce().Average();

            return list.ImputeWithConstant((int)mean);
        }

        public static IEnumerable<long> ImputeWithMean(this IEnumerable<long?> source)
        {
            var list = source.ToList();
            var mean = list.Coalesce().Average();

            return list.ImputeWithConstant((long)mean);
        }

        public static IEnumerable<decimal> ImputeWithMean<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal?> selector)
        {
            return source.Select(selector).ImputeWithMean();
        }

        public static IEnumerable<double> ImputeWithMean<TSource>(this IEnumerable<TSource> source, Func<TSource, double?> selector)
        {
            return source.Select(selector).ImputeWithMean();
        }

        public static IEnumerable<float> ImputeWithMean<TSource>(this IEnumerable<TSource> source, Func<TSource, float?> selector)
        {
            return source.Select(selector).ImputeWithMean();
        }

        public static IEnumerable<int> ImputeWithMean<TSource>(this IEnumerable<TSource> source, Func<TSource, int?> selector)
        {
            return source.Select(selector).ImputeWithMean();
        }

        public static IEnumerable<long> ImputeWithMean<TSource>(this IEnumerable<TSource> source, Func<TSource, long?> selector)
        {
            return source.Select(selector).ImputeWithMean();
        }
    }
}
