﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: MedianCenter.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Transformation
{
    public static partial class Scaling
    {
        public static IEnumerable<decimal> MedianCenter(this IEnumerable<decimal> source)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Shift(-mean);
        }

        public static IEnumerable<double> MedianCenter(this IEnumerable<double> source)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Shift(-mean);
        }

        public static IEnumerable<float> MedianCenter(this IEnumerable<float> source)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Shift(-mean);
        }

        public static IEnumerable<int> MedianCenter(this IEnumerable<int> source)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Shift(-(int)mean);
        }

        public static IEnumerable<long> MedianCenter(this IEnumerable<long> source)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Shift(-(long)mean);
        }

        public static IEnumerable<decimal> MedianCenter<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector)
        {
            return source.Select(selector).MedianCenter();
        }

        public static IEnumerable<double> MedianCenter<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector)
        {
            return source.Select(selector).MedianCenter();
        }

        public static IEnumerable<float> MedianCenter<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector)
        {
            return source.Select(selector).MedianCenter();
        }

        public static IEnumerable<int> MedianCenter<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector)
        {
            return source.Select(selector).MedianCenter();
        }

        public static IEnumerable<long> MedianCenter<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector)
        {
            return source.Select(selector).MedianCenter();
        }
    }
}
