﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: ScaleMeanTo.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;

namespace BOL.Linq.Transformation
{
    public static partial class Scaling
    {
        public static IEnumerable<decimal> ScaleMeanTo(this IEnumerable<decimal> source, decimal factor)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Scale(factor / mean);
        }

        public static IEnumerable<double> ScaleMeanTo(this IEnumerable<double> source, double factor)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Scale(factor / mean);
        }

        public static IEnumerable<float> ScaleMeanTo(this IEnumerable<float> source, float factor)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Scale(factor / mean);
        }

        public static IEnumerable<int> ScaleMeanTo(this IEnumerable<int> source, int factor)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Scale((int)(factor / mean));
        }

        public static IEnumerable<long> ScaleMeanTo(this IEnumerable<long> source, long factor)
        {
            var list = source.ToList();
            var mean = list.Average();

            return list.Scale((long)(factor / mean));
        }

        public static IEnumerable<decimal> ScaleMeanTo<TSource>(this IEnumerable<TSource> source, Func<TSource, decimal> selector, decimal factor)
        {
            return source.Select(selector).ScaleMeanTo(factor);
        }

        public static IEnumerable<double> ScaleMeanTo<TSource>(this IEnumerable<TSource> source, Func<TSource, double> selector, double factor)
        {
            return source.Select(selector).ScaleMeanTo(factor);
        }

        public static IEnumerable<float> ScaleMeanTo<TSource>(this IEnumerable<TSource> source, Func<TSource, float> selector, float factor)
        {
            return source.Select(selector).ScaleMeanTo(factor);
        }

        public static IEnumerable<int> ScaleMeanTo<TSource>(this IEnumerable<TSource> source, Func<TSource, int> selector, int factor)
        {
            return source.Select(selector).ScaleMeanTo(factor);
        }

        public static IEnumerable<long> ScaleMeanTo<TSource>(this IEnumerable<TSource> source, Func<TSource, long> selector, long factor)
        {
            return source.Select(selector).ScaleMeanTo(factor);
        }
    }
}
