﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: TwoSampleTTest.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2012-09-06 11:39 AM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System;
using System.Collections.Generic;
using System.Linq;
using BOL.Linq.Descriptive;
using BOL.Maths.Distributions;

namespace BOL.Maths.Statistics.HypothesisTest
{
    public sealed class TwoSampleTTest : TwoSampleMeanTestBase
    {
        #region Constructors

        public TwoSampleTTest(TestType type, double alpha, double mu12, IEnumerable<double> sample1, IEnumerable<double> sample2)
            : base(type, alpha, mu12, sample1, sample2)
        {
            Distribution = new StudentsTDistribution(SampleSize1 + SampleSize2 - 2);
        }

        public TwoSampleTTest(TestType type, double mu12, IEnumerable<double> sample1, IEnumerable<double> sample2) : this(type, 0.05, mu12, sample1, sample2) { }

        public TwoSampleTTest(double alpha, double mu12, IEnumerable<double> sample1, IEnumerable<double> sample2) : this(TestType.TwoSided, alpha, mu12, sample1, sample2) { }

        public TwoSampleTTest(double mu12, IEnumerable<double> sample1, IEnumerable<double> sample2) : this(TestType.TwoSided, 0.05, mu12, sample1, sample2) { }

        #endregion

        #region Protected methods

        protected override double MeanDifferenceStandardError()
        {
            return Math.Sqrt((1.0 / SampleSize1 + 1.0 / SampleSize2) * ((SampleSize1 - 1) * Sample1.Variance() + 
                (SampleSize2 - 1) * Sample2.Variance()) / (SampleSize1 + SampleSize2 - 2));
        }

        #endregion
    }

    public sealed class PariedTwoSampleTTest : OneSampleTTest
    {
        #region Constructor

        public PariedTwoSampleTTest(TestType type, double alpha, double mu12, IEnumerable<double> sample1, IEnumerable<double> sample2)
            : base(type, alpha, mu12, sample1.Zip(sample2, (s1, s2) => s1 - s2)) { }

        #endregion
    }
}
