﻿#region Copyright information
// 
// Copyright © 2005-2013 Yongkee Cho. All rights reserved.
// 
// This code is a part of the Biological Object Library and governed under the terms of the
// GNU Lesser General  Public License (LGPL) version 2.1 which accompanies this distribution.
// For more information on the LGPL, please visit http://bol.codeplex.com/license.
// 
// - Filename: GeneType.cs
// - Author: Yongkee Cho
// - Email: yongkeecho@gmail.com
// - Date Created: 2013-01-25 3:25 PM
// - Last Modified: 2013-01-25 3:59 PM
// 
#endregion
using System.Runtime.Serialization;

namespace BOL.Models.Features
{
    /// <summary>
    /// Specifies type of gene.
    /// </summary>
    [DataContract]
    public enum GeneType
    {
        /// <summary>Protein coding gene.</summary>
        [EnumMember]
        ProteinCoding,

        /// <summary>Pseudogene.</summary>
        [EnumMember]
        Pseudo,

        /// <summary>Transfer RNA, tRNA.</summary>
        [EnumMember]
        TransferRna,

        /// <summary>Ribosomal RNA, rRNA.</summary>
        [EnumMember]
        RibosomalRna,

        /// <summary>Small nuclear RNA, snRNA.</summary>
        [EnumMember]
        SmallNuclearRna,

        /// <summary>Small cytoplasmic RNA, scRNA.</summary>
        [EnumMember]
        SmallCytoplasmicRna,

        /// <summary>Small nucleolar RNA, snoRNA.</summary>
        [EnumMember]
        SmallNucleolarRna,

        /// <summary>Miscellaneous RNA, miscRNA</summary>
        [EnumMember]
        MiscellaneousRna,

        /// <summary>Non-coding RNA, ncRNA</summary>
        [EnumMember]
        NonCodingRna,

        /// <summary>Other (when the type is known, but there is no specific enumeration for it)</summary>
        [EnumMember]
        Other,

        /// <summary>Unknown (when the type of gene is uncertain)</summary>
        [EnumMember]
        Unknown
    }
}
